#!/usr/bin/env python

import pyopencl as cl
import time
import ctypes
import struct

ctx = cl.create_some_context()
queue = cl.CommandQueue(ctx)
prg = cl.Program(ctx, """

#pragma OPENCL EXTENSION cl_khr_global_int32_base_atomics : enable

typedef unsigned int u32;

u32 strlen(const char *str)
{
  const char *s;
  for (s = str; *s; ++s) {}
  return(s - str);
}

void reverse(char s[])
{
  u32 i, j;
  char c;

  for (i = 0, j = strlen(s)-1; i<j; i++, j--) {
    c = s[i];
    s[i] = s[j];
    s[j] = c;
  }
}

void itoa(char s[], u32 n)
{
  u32 i = 0;
  do {
    s[i++] = n % 10 + '0';
  } while ((n /= 10) > 0);
  s[i] = 0;
  reverse(s);
}
 
u32 length_of_number(u32 n)
{
  char s[16];
  itoa(s, n);
  return strlen(s);
}

__kernel void sum(__global u32* global_sum)
{
  u32 gid = get_global_id(0);
  atomic_add(global_sum, length_of_number(gid));
}

""").build()

start_time = time.time()

# Create 32-bit destination buffer on device and initialize it to zero.
result_d = cl.Buffer(ctx, cl.mem_flags.WRITE_ONLY, 4)
cl.enqueue_fill_buffer(queue, result_d, ctypes.create_string_buffer(1), 0, 4, wait_for=True)

# Launch the kernel.
prg.sum(queue, (0xffffff, ), None, result_d)

# Create the host result buffer and copy the device result there.
result_h = ctypes.create_string_buffer(4)
cl.enqueue_copy(queue, result_h, result_d)

# Convert the host result buffer to a 32-bit unsigned int and print it.
print struct.unpack('I', result_h)[0]

print int((time.time() - start_time) * 1000), 'ms'
